import * as FS from "fs";
import * as Path from "path";
import * as Utility from "util";
import * as Process from "process";

/***
 * Promisified Version of {@link FS.cp}
 * ---
 *
 * Asynchronously copies the entire directory structure from source to destination, including subdirectories and files.
 * - When copying a directory to another directory, globs are not supported.
 *
 * @experimental
 *
 * @param source {typeof import("fs").PathOrFileDescriptor} source path to copy.
 * @param target {typeof import("fs").PathOrFileDescriptor} destination path to copy to.
 * @returns {Promise<?>}
 *
 * @constructor
 *
 */

const Copy = async (source, target) => {
    const $ = async () => await new Promise((resolve) => {
        FS.cp(Path.resolve(source), Path.resolve(target), {
                recursive: true,
                dereference: true,
                preserveTimestamps: false,
                errorOnExist: false,
                force: true
            }, () => resolve
        );
    })

    await $();
}

export { Copy };
export default { Copy };

const Arguments = Process.argv.splice(2);


const Source = (Arguments.includes("--source")) ? Arguments[Arguments.indexOf("--source") + 1] ?? null : null;
const Target = (Arguments.includes("--target")) ? Arguments[Arguments.indexOf("--target") + 1] ?? null : null;

(Target && Source) && (async () => await Copy(Source, Target))();

(Target == null && Source == null) && (async () => await Copy("node_modules", "nodejs/node_modules"))();
